#! /usr/bin/perl
#---------------------------------------------------------------------
# $Id: testPrereq.pl 4815 2011-11-20 06:59:43Z cjm $
# Copyright 2006 Christopher J. Madsen
#
# Make sure we have all the prerequisites for the FFV test suite
#---------------------------------------------------------------------

use strict;
use File::Temp 'tempdir';
use FindBin    '$Bin';
use Test::More;

# Make sure rndfile was built ok:

my $rndfile = `"$Bin/rndfile" --version 2>&1`;

unless ($? == 0) {
  my $signal = $? & 0xFF;
  my $status = $? >> 8;

  die "Error running rndfile (exit $status, signal $signal)";
} # end unless no error running rndfile

unless ($rndfile =~ /rndfile version \d+ by Christopher J\. Madsen/) {
  die "The '$Bin/rndfile' program doesn't appear to be the one from FFV";
} # end unless the --version output seems to be correct

# Make sure we've got the 'prove' command from Test::More:

my $prove = `prove --version 2>&1`;

unless ($? == 0) {
  my $signal = $? & 0xFF;
  my $status = $? >> 8;

  die "Error running prove (exit $status, signal $signal)";
} # end unless no error running prove

unless ($prove =~ /(?:Test|TAP)::.+ Perl/) {
  die "The 'prove' command doesn't appear to be the one from Test::More or TAP::Harness";
} # end unless the --version output seems to be correct

# If we got here, then it looks like we've got everything we need:
exit 0;
